@@ -21,17 +21,25 @@
  *
  *****************************************************************************/
 
+#include "config.h"
 #include "utils.h"
-#include "crypt.h"
+#include "gm_crypt.h"
 #include "base64.h"
 #include "gearman.h"
 #include "popenRWE.h"
 #include "polarssl/md5.h"
 
-pid_t current_child_pid = 0;
 char temp_buffer1[GM_BUFFERSIZE];
 char temp_buffer2[GM_BUFFERSIZE];
 
+#ifdef EMBEDDEDPERL
+#include "epn_utils.h"
+int enable_embedded_perl         = GM_ENABLED;
+int use_embedded_perl_implicitly = GM_DISABLED;
+int use_perl_cache               = GM_ENABLED;
+char *p1_file                    = NULL;
+#endif
+
 /* escapes newlines in a string */
 char *gm_escape_newlines(char *rawbuf, int trimmed) {
     char *tmpbuf=NULL;
@@ -236,7 +244,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->max_worker         = GM_DEFAULT_MAX_WORKER;
     opt->transportmode      = GM_ENCODE_AND_ENCRYPT;
     opt->daemon_mode        = GM_DISABLED;
-    opt->fork_on_exec       = GM_ENABLED;
+    opt->fork_on_exec       = GM_DISABLED;
     opt->idle_timeout       = GM_DEFAULT_IDLE_TIMEOUT;
     opt->max_jobs           = GM_DEFAULT_MAX_JOBS;
     opt->spawn_rate         = GM_DEFAULT_SPAWN_RATE;
@@ -248,6 +256,13 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->host               = NULL;
     opt->service            = NULL;
 
+#ifdef EMBEDDEDPERL
+    opt->enable_embedded_perl         = GM_ENABLED;
+    opt->use_embedded_perl_implicitly = GM_DISABLED;
+    opt->use_perl_cache               = GM_ENABLED;
+    opt->p1_file                      = NULL;
+#endif
+
     opt->server_num         = 0;
     for(i=0;i<=GM_LISTSIZE;i++)
         opt->server_list[i] = NULL;
@@ -404,6 +419,31 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         return(GM_OK);
     }
 
+    /* enable_embedded_perl */
+    else if ( !strcmp( key, "enable_embedded_perl" ) ) {
+#ifdef EMBEDDEDPERL
+        opt->enable_embedded_perl = parse_yes_or_no(value, GM_ENABLED);
+        enable_embedded_perl = opt->enable_embedded_perl;
+#endif
+        return(GM_OK);
+    }
+    /* use_embedded_perl_implicitly */
+    else if ( !strcmp( key, "use_embedded_perl_implicitly" ) ) {
+#ifdef EMBEDDEDPERL
+        opt->use_embedded_perl_implicitly = parse_yes_or_no(value, GM_ENABLED);
+        use_embedded_perl_implicitly = opt->use_embedded_perl_implicitly;
+#endif
+        return(GM_OK);
+    }
+    /* use_perl_cache */
+    else if ( !strcmp( key, "use_perl_cache" ) ) {
+#ifdef EMBEDDEDPERL
+        opt->use_perl_cache = parse_yes_or_no(value, GM_ENABLED);
+        use_perl_cache = opt->use_perl_cache;
+#endif
+        return(GM_OK);
+    }
+
     else if ( value == NULL ) {
         gm_log( GM_LOG_ERROR, "unknown switch '%s'\n", key );
         return(GM_OK);
@@ -719,6 +759,15 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         }
     }
 
+    /* p1_file */
+    else if ( !strcmp( key, "p1_file" ) ) {
+#ifdef EMBEDDEDPERL
+        opt->p1_file = strdup( value );
+        free(p1_file);
+        p1_file = strdup(opt->p1_file);
+#endif
+    }
+
     else {
         gm_log( GM_LOG_ERROR, "unknown option '%s'\n", key );
     }
@@ -805,6 +854,16 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
         gm_log( GM_LOG_DEBUG, "max worker:          %d\n", opt->max_worker);
         gm_log( GM_LOG_DEBUG, "spawn rate:          %d\n", opt->spawn_rate);
         gm_log( GM_LOG_DEBUG, "fork on exec:        %s\n", opt->fork_on_exec == GM_ENABLED ? "yes" : "no");
+#ifndef EMBEDDEDPERL
+        gm_log( GM_LOG_DEBUG, "embedded perl:       not compiled\n");
+#endif
+#ifdef EMBEDDEDPERL
+        gm_log( GM_LOG_DEBUG, "\n" );
+        gm_log( GM_LOG_DEBUG, "embedded perl:       %s\n", opt->enable_embedded_perl == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "use_epn_implicitly:  %s\n", opt->use_embedded_perl_implicitly == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "use_perl_cache:      %s\n", opt->use_perl_cache == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "p1_file:             %s\n", opt->p1_file == NULL ? "not set" : opt->p1_file );
+#endif
     }
     if(mode == GM_NEB_MODE) {
         gm_log( GM_LOG_DEBUG, "debug result:        %s\n", opt->debug_result == GM_ENABLED ? "yes" : "no");
@@ -871,6 +930,8 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
 
 /* free options structure */
 void mod_gm_free_opt(mod_gm_opt_t *opt) {
+    if(opt == NULL)
+        return;
     int i,j;
     for(i=0;i<opt->server_num;i++)
         free(opt->server_list[i]);
@@ -898,7 +959,12 @@ void mod_gm_free_opt(mod_gm_opt_t *opt) {
     free(opt->host);
     free(opt->service);
     free(opt->identifier);
+#ifdef EMBEDDEDPERL
+    free(opt->p1_file);
+#endif
     free(opt);
+    opt=NULL;
+    return;
 }
 
 
@@ -927,52 +993,6 @@ int read_keyfile(mod_gm_opt_t *opt) {
     return(GM_OK);
 }
 
-
-/* convert number to signal name */
-char * nr2signal(int sig) {
-    char * signame = NULL;
-    switch(sig) {
-        case 1:  signame = "SIGHUP";
-                 break;
-        case 2:  signame = "SIGINT";
-                 break;
-        case 3:  signame = "SIGQUIT";
-                 break;
-        case 4:  signame = "SIGILL";
-                 break;
-        case 5:  signame = "SIGTRAP";
-                 break;
-        case 6:  signame = "SIGABRT";
-                 break;
-        case 7:  signame = "SIGBUS";
-                 break;
-        case 8:  signame = "SIGFPE";
-                 break;
-        case 9:  signame = "SIGKILL";
-                 break;
-        case 10: signame = "SIGUSR1";
-                 break;
-        case 11: signame = "SIGSEGV";
-                 break;
-        case 12: signame = "SIGUSR2";
-                 break;
-        case 13: signame = "SIGPIPE";
-                 break;
-        case 14: signame = "SIGALRM";
-                 break;
-        case 15: signame = "SIGTERM";
-                 break;
-        case 16: signame = "SIGURG";
-                 break;
-        default: signame = malloc(20);
-                 snprintf(signame, 20, "signal %d", sig);
-                 return signame;
-                 break;
-    }
-    return strdup(signame);
-}
-
-
 /* convert to time */
 void string2timeval(char * value, struct timeval *t) {
     char * v;
@@ -1032,350 +1052,6 @@ long mod_gm_time_compare(struct timeval * tv1, struct timeval * tv2) {
 }
 
 
-/* extract check result */
-char *extract_check_result(FILE *fp, int trimmed) {
-    int size;
-    char buffer[GM_BUFFERSIZE] = "";
-    char output[GM_BUFFERSIZE] = "";
-
-    /* get all lines of plugin output - escape newlines */
-    strcpy(buffer,"");
-    strcpy(output,"");
-    size = GM_MAX_OUTPUT;
-    while(size > 0 && fgets(buffer,sizeof(buffer)-1,fp)){
-        strncat(output, buffer, size);
-        size -= strlen(buffer);
-    }
-
-    return(gm_escape_newlines(output, trimmed));
-}
-
-
-/* convert a command line to an array of arguments, suitable for exec* functions */
-int parse_command_line(char *cmd, char *argv[MAX_CMD_ARGS]) {
-    unsigned int argc=0;
-    char *parsed_cmd;
-
-    /* Skip initial white-space characters. */
-    for(parsed_cmd=cmd;isspace(*cmd);++cmd)
-        ;
-
-    /* Parse command line. */
-    while(*cmd&&(argc<MAX_CMD_ARGS-1)){
-        argv[argc++]=parsed_cmd;
-        switch(*cmd){
-        case '\'':
-            while((*cmd)&&(*cmd!='\''))
-                *(parsed_cmd++)=*(cmd++);
-            if(*cmd)
-                ++cmd;
-            break;
-        case '"':
-            while((*cmd)&&(*cmd!='"')){
-                if((*cmd=='\\')&&cmd[1]&&strchr("\"\\\n",cmd[1]))
-                    ++cmd;
-                *(parsed_cmd++)=*(cmd++);
-                }
-            if(*cmd)
-                ++cmd;
-            break;
-        default:
-            while((*cmd)&&!isspace(*cmd)){
-                if((*cmd=='\\')&&cmd[1])
-                    ++cmd;
-                *(parsed_cmd++)=*(cmd++);
-                }
-            }
-        while(isspace(*cmd))
-            ++cmd;
-        *(parsed_cmd++)='\0';
-        }
-    argv[argc]=NULL;
-
-    return GM_OK;
-}
-
-
-/* run a check */
-int run_check(char *processed_command, char **ret, char **err) {
-    char *argv[MAX_CMD_ARGS];
-    FILE *fp;
-    pid_t pid;
-    int pipe_stdout[2], pipe_stderr[2], pipe_rwe[3];
-    int retval;
-
-    /* check for check execution method (shell or execvp) */
-    if(!strpbrk(processed_command,"!$^&*()~[]|{};<>?`\"'")) {
-        /* use the fast execvp when there are now shell characters */
-        gm_log( GM_LOG_TRACE, "using execvp\n" );
-
-        parse_command_line(processed_command,argv);
-        if(!argv[0])
-            _exit(STATE_UNKNOWN);
-
-        if(pipe(pipe_stdout)) {
-            gm_log( GM_LOG_ERROR, "error creating pipe: %s\n", strerror(errno));
-            _exit(STATE_UNKNOWN);
-        }
-        if(pipe(pipe_stderr)) {
-            gm_log( GM_LOG_ERROR, "error creating pipe: %s\n", strerror(errno));
-            _exit(STATE_UNKNOWN);
-        }
-        if((pid=fork())<0){
-            gm_log( GM_LOG_ERROR, "fork error\n");
-            _exit(STATE_UNKNOWN);
-        }
-        else if(!pid){
-            /* child process */
-            if((dup2(pipe_stdout[1],STDOUT_FILENO)<0)){
-                gm_log( GM_LOG_ERROR, "dup2 error\n");
-                _exit(STATE_UNKNOWN);
-            }
-            if((dup2(pipe_stderr[1],STDERR_FILENO)<0)){
-                gm_log( GM_LOG_ERROR, "dup2 error\n");
-                _exit(STATE_UNKNOWN);
-            }
-            close(pipe_stdout[1]);
-            close(pipe_stderr[1]);
-            current_child_pid = getpid();
-            execvp(argv[0], argv);
-            if(errno == 2)
-                _exit(127);
-            if(errno == 13)
-                _exit(126);
-            _exit(STATE_UNKNOWN);
-        }
-
-        /* parent */
-        /* prepare stdout pipe reading */
-        close(pipe_stdout[1]);
-        fp=fdopen(pipe_stdout[0],"r");
-        if(!fp){
-            gm_log( GM_LOG_ERROR, "fdopen error\n");
-            _exit(STATE_UNKNOWN);
-        }
-        *ret = extract_check_result(fp, GM_DISABLED);
-        fclose(fp);
-
-        /* prepare stderr pipe reading */
-        close(pipe_stderr[1]);
-        fp=fdopen(pipe_stderr[0],"r");
-        if(!fp){
-            gm_log( GM_LOG_ERROR, "fdopen error\n");
-            _exit(STATE_UNKNOWN);
-        }
-        *err = extract_check_result(fp, GM_ENABLED);
-        fclose(fp);
-
-        close(pipe_stdout[0]);
-        close(pipe_stderr[0]);
-        if(waitpid(pid,&retval,0)!=pid)
-            retval=-1;
-    }
-    else {
-        /* use the slower popen when there were shell characters */
-        gm_log( GM_LOG_TRACE, "using popen\n" );
-        current_child_pid = getpid();
-        pid = popenRWE(pipe_rwe, processed_command);
-
-        /* extract check result */
-        fp=fdopen(pipe_rwe[1],"r");
-        if(!fp){
-            gm_log( GM_LOG_ERROR, "fdopen error\n");
-            _exit(STATE_UNKNOWN);
-        }
-        *ret = extract_check_result(fp, GM_DISABLED);
-        fclose(fp);
-
-        /* extract check stderr */
-        fp=fdopen(pipe_rwe[2],"r");
-        if(!fp){
-            gm_log( GM_LOG_ERROR, "fdopen error\n");
-            _exit(STATE_UNKNOWN);
-        }
-        *err = extract_check_result(fp, GM_ENABLED);
-        fclose(fp);
-
-        /* close the process */
-        retval=pcloseRWE(pid, pipe_rwe);
-    }
-
-    return retval;
-}
-
-
-/* execute this command with given timeout */
-int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier) {
-    int pipe_stdout[2] , pipe_stderr[2];
-    int return_code;
-    int pclose_result;
-    char *plugin_output, *plugin_error;
-    char *bufdup;
-    char buffer[GM_BUFFERSIZE], buf_error[GM_BUFFERSIZE];
-    sigset_t mask;
-    struct timeval start_time,end_time;
-    pid_t pid    = 0;
-    buffer[0]    = '\x0';
-    buf_error[0] = '\x0';
-
-    gm_log( GM_LOG_TRACE, "execute_safe_command()\n" );
-
-    if(exec_job->start_time.tv_sec == 0) {
-        gettimeofday(&start_time,NULL);
-        exec_job->start_time = start_time;
-    }
-
-    /* fork a child process */
-    if(fork_exec == GM_ENABLED) {
-        if(pipe(pipe_stdout) != 0)
-            perror("pipe stdout");
-        if(pipe(pipe_stderr) != 0)
-            perror("pipe stderr");
-
-        pid=fork();
-
-        /*fork error */
-        if( pid == -1 ) {
-            exec_job->output      = strdup("(Error On Fork)");
-            exec_job->return_code = 3;
-            return(GM_ERROR);
-        }
-    }
-
-    /* we are in the child process */
-    if( fork_exec == GM_DISABLED || pid == 0 ) {
-
-        /* become the process group leader */
-        setpgid(0,0);
-        pid = getpid();
-
-        /* remove all customn signal handler */
-        sigfillset(&mask);
-        sigprocmask(SIG_UNBLOCK, &mask, NULL);
-
-        if( fork_exec == GM_ENABLED ) {
-            close(pipe_stdout[0]);
-            close(pipe_stderr[0]);
-        }
-        signal(SIGALRM, check_alarm_handler);
-        alarm(exec_job->timeout);
-
-        /* run the plugin check command */
-        pclose_result = run_check(exec_job->command_line, &plugin_output, &plugin_error);
-        return_code   = pclose_result;
-
-        if(fork_exec == GM_ENABLED) {
-            if(write(pipe_stdout[1], plugin_output, strlen(plugin_output)+1) <= 0)
-                perror("write stdout");
-            if(write(pipe_stderr[1], plugin_error, strlen(plugin_error)+1) <= 0)
-                perror("write");
-
-            if(pclose_result == -1) {
-                char error[GM_BUFFERSIZE];
-                snprintf(error, sizeof(error), "error on %s: %s", identifier, strerror(errno));
-                if(write(pipe_stdout[1], error, strlen(error)+1) <= 0)
-                    perror("write");
-            }
-
-            return_code = real_exit_code(pclose_result);
-            free(plugin_output);
-            free(plugin_error);
-            exit(return_code);
-        }
-        else {
-            snprintf( buffer,    sizeof( buffer )-1,    "%s", plugin_output );
-            snprintf( buf_error, sizeof( buf_error )-1, "%s", plugin_error  );
-            free(plugin_output);
-            free(plugin_error);
-        }
-    }
-
-    /* we are the parent */
-    if( fork_exec == GM_DISABLED || pid > 0 ){
-
-        gm_log( GM_LOG_TRACE, "started check with pid: %d\n", pid);
-
-        if( fork_exec == GM_ENABLED) {
-            close(pipe_stdout[1]);
-            close(pipe_stderr[1]);
-
-            waitpid(pid, &return_code, 0);
-            gm_log( GM_LOG_TRACE, "finished check from pid: %d with status: %d\n", pid, return_code);
-            /* get all lines of plugin output */
-            if(read(pipe_stdout[0], buffer, sizeof(buffer)-1) < 0)
-                perror("read");
-            if(read(pipe_stderr[0], buf_error, sizeof(buf_error)-1) < 0)
-                perror("read");
-        }
-        return_code = real_exit_code(return_code);
-
-        /* file not executable? */
-        if(return_code == 126) {
-            return_code = STATE_CRITICAL;
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 126 is out of bounds. Make sure the plugin you're trying to run is executable. (worker: %s)", identifier);
-        }
-        /* file not found errors? */
-        else if(return_code == 127) {
-            return_code = STATE_CRITICAL;
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of 127 is out of bounds. Make sure the plugin you're trying to run actually exists. (worker: %s)", identifier);
-        }
-        /* signaled */
-        else if(return_code >= 128 && return_code < 144) {
-            char * signame = nr2signal((int)(return_code-128));
-            bufdup = strdup(buffer);
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)", (int)(return_code), signame, identifier);
-            if(strlen(bufdup) > 0) {
-                strncat(buffer, "\\n", (sizeof(buffer)-1));
-                strncat(buffer, bufdup, (sizeof(buffer)-1));
-            }
-            return_code = STATE_CRITICAL;
-            free(bufdup);
-            free(signame);
-        }
-        /* other error codes > 3 */
-        else if(return_code > 3) {
-            gm_log( GM_LOG_INFO, "check exited with exit code > 3. Exit: %d\n", (int)(return_code));
-            gm_log( GM_LOG_INFO, "stdout: %s\n", buffer);
-            bufdup = strdup(buffer);
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. (worker: %s)\\n%s", (int)(return_code), identifier, bufdup);
-            free(bufdup);
-            if(return_code != 25 && mod_gm_opt->workaround_rc_25 == GM_DISABLED) {
-                return_code = STATE_CRITICAL;
-            }
-        }
-
-        exec_job->output      = strdup(buffer);
-        exec_job->error       = strdup(buf_error);
-        exec_job->return_code = return_code;
-        if( fork_exec == GM_ENABLED) {
-            close(pipe_stdout[0]);
-            close(pipe_stderr[0]);
-        }
-    }
-    alarm(0);
-    current_child_pid = 0;
-    pid               = 0;
-
-    /* record check result info */
-    gettimeofday(&end_time, NULL);
-    exec_job->finish_time = end_time;
-
-    /* did we have a timeout? */
-    if(exec_job->timeout < ((int)end_time.tv_sec - (int)exec_job->start_time.tv_sec)) {
-        exec_job->return_code   = 2;
-        exec_job->early_timeout = 1;
-        if ( !strcmp( exec_job->type, "service" ) )
-            snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)", identifier);
-        if ( !strcmp( exec_job->type, "host" ) )
-            snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)", identifier);
-        free(exec_job->output);
-        exec_job->output = strdup( buffer );
-    }
-
-    return(GM_OK);
-}
-
-
 /* set empty default job */
 int set_default_job(gm_job_t *job, mod_gm_opt_t *opt) {
 
@@ -1416,43 +1092,6 @@ int free_job(gm_job_t *job) {
 }
 
 
-
-/* called when check runs into timeout */
-void check_alarm_handler(int sig) {
-    int retval;
-    pid_t pid = getpid();
-
-    gm_log( GM_LOG_TRACE, "check_alarm_handler(%i)\n", sig );
-    if(current_job != NULL && mod_gm_opt->fork_on_exec == GM_DISABLED) {
-        send_timeout_result(current_job);
-        gearman_job_send_complete(current_gearman_job, NULL, 0);
-    }
-
-    if(current_child_pid > 0) {
-        gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", current_child_pid);
-        signal(SIGINT, SIG_IGN);
-        kill(-current_child_pid, SIGINT);
-        sleep(1);
-        if(waitpid(pid,&retval,0)!=pid)
-            return;
-        if(pid_alive(current_child_pid)) {
-            gm_log( GM_LOG_TRACE, "send SIGKILL to %d\n", current_child_pid);
-            kill(-current_child_pid, SIGKILL);
-        }
-    } else {
-        signal(SIGINT, SIG_IGN);
-        gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", pid);
-        kill(-pid, SIGINT);
-        signal(SIGINT, SIG_DFL);
-        sleep(1);
-        gm_log( GM_LOG_TRACE, "send SIGKILL to %d\n", pid);
-        kill(-pid, SIGKILL);
-    }
-
-    return;
-}
-
-
 /* verify if a pid is alive */
 int pid_alive(int pid) {
     if(pid < 0) { pid = -pid; }
@@ -1871,7 +1510,7 @@ void gm_log( int lvl, const char *text, ... ) {
         level  = "DEBUG";
         slevel = LOG_DEBUG;
     }
-    else if ( lvl == GM_LOG_TRACE ) {
+    else if ( lvl >= GM_LOG_TRACE ) {
         level  = "TRACE";
         slevel = LOG_DEBUG;
     }
@@ -1944,30 +1583,6 @@ char * get_param_server(char * servername, char * server_list[GM_LISTSIZE], int
     return new_server;
 }
 
-void send_timeout_result(gm_job_t * exec_job) {
-    struct timeval end_time;
-    char buffer[GM_BUFFERSIZE];
-    buffer[0] = '\x0';
-
-    gm_log( GM_LOG_TRACE, "send_timeout_result()\n");
-
-    gettimeofday(&end_time, NULL);
-    exec_job->finish_time = end_time;
-
-    exec_job->return_code   = 2;
-    exec_job->early_timeout = 1;
-    if ( !strcmp( exec_job->type, "service" ) )
-        snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
-    if ( !strcmp( exec_job->type, "host" ) )
-        snprintf( buffer, sizeof( buffer ) -1, "(Host Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
-    free(exec_job->output);
-    exec_job->output = strdup( buffer );
-
-    send_result_back(exec_job);
-
-    return;
-}
-
 
 /* send results back */
 void send_result_back(gm_job_t * exec_job) {
@@ -2099,3 +1714,4 @@ char *md5sum(char *text) {
 
     return result;
 }
+
